/******************************************************************
 *
 *  EE-164 example code
 * 
 *  This ADSP-2191 example illustrates the no-boot option (8-bit)
 *
 *  After reset it start program execution from external EPROM
 *  Logical address 0x01.0000 equals to byte address 0x04.0000.
 *  The timer 0 interrupt is serviced by IVint4 residing at
 *  address 0x01.0080.
 *
 *  The example provides an alternative service routine IVint5
 *  which is never executed in EPROM but copied manually to
 *  on-chip memory location 0x00.0080. As soon as the RMODE bit
 *  in the SYSCR register is set, a timer interrupt makes the
 *  core jump to on-chip address 0x00.0080. 
 *
 *  By toggeling the RMODE bit this example blinks either 
 *  LED8 (Flag 0) or LED9 (Flag 1). 
 *
 *  ANALOG DEVICES INC. (bk) 2002
 *
 ******************************************************************/


#include 	<def2191.h>
  

.section / data    data1;
.var isrcnt = 0;

/* PM Reset interrupt vector code */
.section / code IVreset; 
  	jump _main;
  	

/* main program resides in EPROM */  	
.section / code program;

_main:	

/* copy IVT to on-chip memory */
	call copyisr;

/***** init variables *****/

	dmpg1 = page(isrcnt);
	ar = -20;
	dm(isrcnt) = ar;

/***** setup interrupts *****/

	iopg = Interrupt_Controller_Page; 			/* effect latency */

	icntl = 0x0000;						/* disable all interrupts and nesting */

	ar = 0xbbbb;					/* set all Priorties to 15 - mask interrupt source */
	io(IPR0) = ar;
	io(IPR1) = ar;
	io(IPR2) = ar;
	io(IPR3) = ar;
	
	ar = 0xbb0b;
	io(IPR2) = ar;
		
	irptl = 0x0000;					/* clear all IRQ latches */
	nop;
	imask = 0x0010;					/* enable IRQ 4 */

	ena int;
	
	


/***** PROGRAM FLAGIO SETTINGS *****/

	iopg = General_Purpose_IO; 			/* effect latency */

	ar = 0x000F; 	io(DIR)   = ar;		/* PF0..3 are outputs */
	ar = 0x000f;	io(FLAGS)  = ar;	/* clear all LEDs */

	
/***** setup timer 0 for interval ****/

	iopg = Timer_Page;
	
	ar = 0x001D;
	io(CFGR0) = ar;	/* PWM_OUT mode */
	ar = 100;
	io(PRDH0) = ar;
	ar = 0;	
	io(PRDL0) = ar;
	ar = 50;
	io(WHR0) = ar;
	ar = 0;
	io(WLR0) = ar;
	ar = 0x100;
	io(GSR0) = ar;	/* start timer */

	
foreverloop:
	idle;
	jump foreverloop;	
	
		
/********** IVT ****************/

.section / code IVint4;  
	
	ena sr;
	ay0 = 0x0001;

	call isr_core;
	if gt call internal_IVT;
	dm(isrcnt) = ar;

	rti (db);
		iopg = ay1;
		nop;


.section / code IVint5;  
	
	ena sr;
	ay0 = 0x0002;

	/* pay attention to the long (far) branches */
	lcall isr_core;
	if gt lcall external_IVT;
	dm(isrcnt) = ar;

	rti (db);
		iopg = ay1;
		nop;


		
/***** sub functions ****************/	
	
.section / code program;  

/* copy external IV5 to on-chip IV4 */
copyisr:

	iopg = External_Access_Bridge_Page;
	ar = 0x0008;	// set E_DFS
	io(E_STAT) = ar;
	  
	iopg = External_Memory_Interface_Page;
	ar = 0x0070;	// 8-bit as by default
	io(EMICTL) = ar;
	ar = 0x0DFF; // default values
	io(MS0CTL) = ar;	
	
	dmpg1 = 0;
	i0 = IVint4;
	l0 = 0; m0 = 1;
	
	dmpg2 = page(IVint5);
	i4 = IVint5;
	l4 = 0; m4 = 1;
	
	cntr = 32;
	do copyloop-1 until ce;
		ar = pm(i4+=m4);
		pm(i0+=m0) = ar;
copyloop:		
	
	flush cache;
	rts;


		 	
	
isr_core:

	ay1 = iopg;

	/* clear timer irq request */
	iopg = Timer_Page;
	ar = 0x0001;
	io(GSR0) = ar;
	
	/* toggle Flag0 pin */
	
	iopg = General_Purpose_IO;
	
	ar = ax0 and ay0;
	ar = ar xor ay0;
	ax0 = ar;	
	
	if eq jump setflag0;
clearflag0:
		io(FLAGC) = ay0;		
		jump common0;
setflag0:	
		io(FLAGS) = ay0;		
common0:
	
	dmpg1 = page(isrcnt);
	rts (db);
		ar = dm(isrcnt);
		ar = ar + 1;


		
/* switch IVT to on-chip memory */	
internal_IVT:
	iopg = Clock_and_System_Control_Page;
	ar = io(SYSCR);
	ar = setbit 4 of ar;	// set RMODE bit	
	io(SYSCR) = ar;

	dmpg1 = page(isrcnt);
	rts (db);
		ar = -20;
		dm(isrcnt) = ar;
		
/* switch IVT to off-chip memory */	
external_IVT:
	iopg = Clock_and_System_Control_Page;
	ar = io(SYSCR);
	ar = clrbit 4 of ar;	// set RMODE bit	
	io(SYSCR) = ar;

	dmpg1 = page(isrcnt);
	rts (db);
		ar = -20;
		dm(isrcnt) = ar;		

		
		
